package com.kdanmobile.reader

import android.annotation.SuppressLint
import android.arch.lifecycle.Observer
import android.arch.lifecycle.ViewModelProviders
import android.net.Uri
import android.os.Build
import android.support.v7.app.AppCompatActivity
import android.os.Bundle
import android.support.constraint.ConstraintSet
import android.view.Menu
import android.view.View
import com.kdanmobile.kmpdfkit.pdfcommon.KMPDFReaderView
import com.kdanmobile.reader.Utils.applyConstraintSet
import kotlinx.android.synthetic.main.activity_reader.*
import kotlin.Exception

open class ReaderActivity : AppCompatActivity() {

    companion object {
        const val KEY_FILE_ABSOLUTE = "file_absolutepath"
    }

    private lateinit var viewModel: ReaderViewModel

    // For hide/show toolbars
    private var isHideToolbar = false
    private lateinit var originConstraintSet: ConstraintSet

    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        setContentView(R.layout.activity_reader)
        setSupportActionBar(toolbar_readerActivity)

        cloneOriginConstraintSet()
        setupBottomToolbar()

        val factory = ReaderViewModelFactory()
        viewModel = ViewModelProviders.of(this, factory).get(ReaderViewModel::class.java)
        viewModel.isOpenedFileLiveData.observe(this, Observer(this::onIsOpenedFileUpdate))
        viewModel.fileNameLiveData.observe(this, Observer { tv_readerActivity_title.text = it })
        viewModel.isOpenedFileLiveData.value?.also { isOpened ->
            if (isOpened) return@also
            val filePath = intent.getStringExtra(KEY_FILE_ABSOLUTE)
            val uri = Uri.parse(filePath)
            viewModel.openPdfFile(this, uri, intent.type)
        }
    }

    override fun onCreateOptionsMenu(menu: Menu?): Boolean {
        menuInflater.inflate(R.menu.reader_more, menu)
        if (Build.VERSION.SDK_INT < Build.VERSION_CODES.KITKAT) {
            menu?.findItem(R.id.item_reader_more_print)?.isVisible = false
        }
        // TODO: Check is pdf file or epub file, these action support only pdf file
        val isPdf = true
        if (!isPdf) {
            menu?.apply {
                findItem(R.id.item_reader_more_text_reflow)?.isVisible = false
                findItem(R.id.item_reader_more_file_info)?.isVisible = false
                findItem(R.id.item_reader_more_print)?.isVisible = false
            }
        }
        return true
    }

    @SuppressLint("RestrictedApi")
    override fun onPrepareOptionsPanel(view: View?, menu: Menu?): Boolean {
        /** Force show action menu item icon **/
        menu?.also {
            if ("MenuBuilder" != it::javaClass.get().simpleName) return@also
            try {
                val method = it.javaClass.getDeclaredMethod("setOptionalIconsVisible", java.lang.Boolean.TYPE)
                method.isAccessible = true
                method.invoke(it, true)
            } catch (e: Exception) {
                // Do nothing
                println(e.message)
            }
        }
        return super.onPrepareOptionsPanel(view, menu)
    }

    private fun cloneOriginConstraintSet() {
        originConstraintSet = ConstraintSet().apply {
            clone(constrainLayout_readerActivity_root)
        }
    }

    private fun onIsOpenedFileUpdate(isOpened: Boolean?) {
        if (isOpened == null) return
        val container = viewGroup_readerActivity_container
        container.removeAllViews()
        if (!isOpened) return
        val context = this
        val readerView = object : KMPDFReaderView(context) {
            override fun onTapMainDocArea() {
                super.onTapMainDocArea()
                when (isHideToolbar) {
                    true -> showAllToolbars()
                    false -> hideAllToolbars()
                }
                isHideToolbar = !isHideToolbar
            }
        }
        viewModel.setReaderView(readerView)
        container.addView(readerView)
    }

    private fun showAllToolbars() {
        if (Build.VERSION.SDK_INT < Build.VERSION_CODES.KITKAT) {
            // TODO : Implement show toolbars on Android api level < 19 devices
        } else {
            constrainLayout_readerActivity_root.applyConstraintSet(originConstraintSet)
        }
    }

    private fun hideAllToolbars() {
        if (Build.VERSION.SDK_INT < Build.VERSION_CODES.KITKAT) {
            // TODO : Implement hide toolbars on Android api level < 19 devices
        } else {
            constrainLayout_readerActivity_root.applyConstraintSet(this, R.layout.activity_reader_hide_all)
        }
    }

    private fun hideTopLeftBottomToolbars() {
        if (Build.VERSION.SDK_INT < Build.VERSION_CODES.KITKAT) {
            // TODO : Implement hide toolbars on Android api level < 19 devices
        } else {
            constrainLayout_readerActivity_root.applyConstraintSet(this, R.layout.activity_reader_hide_top_left_bottom)
        }
    }

    private fun setupBottomToolbar() {
        val context = this
        /** Set buttons color **/
        val normalColor = R.color.reader_bottom_toolbar_bottom_color_normal
        val pressColor = R.color.reader_bottom_toolbar_bottom_color_press
        Utils.setTintDrawableList(context, ib_readerActivity_bottomToolbarPrevious, R.drawable.ic_chevron_left, normalColor, pressColor)
        Utils.setTintDrawableList(context, ib_readerActivity_bottomToolbarNext, R.drawable.ic_chevron_right, normalColor, pressColor)
        Utils.setTintDrawableList(context, ib_readerActivity_bottomToolbarBookmark, R.drawable.ic_bookmark_border, normalColor, pressColor)
        Utils.setTintDrawableList(context, ib_readerActivity_bottomToolbarViewAll, R.drawable.ic_view_all, normalColor, pressColor)
        Utils.setTintDrawableList(context, ib_readerActivity_bottomToolbarKdanCloud, R.drawable.ic_kdan_cloud, normalColor, pressColor)
    }
}
